@@ -13,20 +13,17 @@
 
 #pragma once
 
-#include <string>
-#include <vector>
+#include <chrono>
 #include <cstdint>
-#include <system_error>
 #include <random>
+#include <string>
+#include <system_error>
+#include <vector>
 #include <locale.h>
 #include <limits.h>
 #include "pplx/pplxtasks.h"
 #include "cpprest/details/basic_types.h"
 
-#if !defined(_WIN32) || (_MSC_VER >= 1700)
-#include <chrono>
-#endif
-
 #ifndef _WIN32
 #include <sys/time.h>
 #if !defined(ANDROID) && !defined(__ANDROID__) && defined(HAVE_XLOCALE_H) // CodePlex 269
@@ -359,13 +356,7 @@ namespace details
     inline bool __cdecl is_alnum(const unsigned char uch) CPPREST_NOEXCEPT
     {   // test if uch is an alnum character
         // special casing char to avoid branches
-        static
-            #if !defined(_MSC_VER) || _MSC_VER >= 1900
-            constexpr
-            #else
-            const
-            #endif
-            bool is_alnum_table[UCHAR_MAX + 1] =
+        static CPPREST_CONSTEXPR bool is_alnum_table[UCHAR_MAX + 1] =
             {
             /*        X0 X1 X2 X3 X4 X5 X6 X7 X8 X9 XA XB XC XD XE XF */
             /* 0X */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -433,7 +424,7 @@ namespace details
     }
 
     /// <summary>
-    /// Cross platform utility function for performing case insensitive string equality comparision.
+    /// Cross platform utility function for performing case insensitive string equality comparison.
     /// </summary>
     /// <param name="left">First string to compare.</param>
     /// <param name="right">Second strong to compare.</param>
@@ -441,7 +432,7 @@ namespace details
     _ASYNCRTIMP bool __cdecl str_iequal(const std::string &left, const std::string &right) CPPREST_NOEXCEPT;
 
     /// <summary>
-    /// Cross platform utility function for performing case insensitive string equality comparision.
+    /// Cross platform utility function for performing case insensitive string equality comparison.
     /// </summary>
     /// <param name="left">First string to compare.</param>
     /// <param name="right">Second strong to compare.</param>
@@ -449,7 +440,7 @@ namespace details
     _ASYNCRTIMP bool __cdecl str_iequal(const std::wstring &left, const std::wstring &right) CPPREST_NOEXCEPT;
 
     /// <summary>
-    /// Cross platform utility function for performing case insensitive string less-than comparision.
+    /// Cross platform utility function for performing case insensitive string less-than comparison.
     /// </summary>
     /// <param name="left">First string to compare.</param>
     /// <param name="right">Second strong to compare.</param>
@@ -457,7 +448,7 @@ namespace details
     _ASYNCRTIMP bool __cdecl str_iless(const std::string &left, const std::string &right) CPPREST_NOEXCEPT;
 
     /// <summary>
-    /// Cross platform utility function for performing case insensitive string less-than comparision.
+    /// Cross platform utility function for performing case insensitive string less-than comparison.
     /// </summary>
     /// <param name="left">First string to compare.</param>
     /// <param name="right">Second strong to compare.</param>
@@ -509,7 +500,7 @@ _ASYNCRTIMP const std::error_category & __cdecl linux_category();
 
 /// <summary>
 /// Gets the one global instance of the current platform's error category.
-/// <summary>
+/// </summary>
 _ASYNCRTIMP const std::error_category & __cdecl platform_category();
 
 /// <summary>
