@@ -26,6 +26,7 @@
 #include "cpprest/asyncrt_utils.h"
 #include "cpprest/streams.h"
 #include "cpprest/containerstream.h"
+#include "cpprest/http_compression.h"
 
 namespace web
 {
@@ -338,6 +339,38 @@ public:
     /// </summary>
     const concurrency::streams::ostream & outstream() const { return m_outStream; }
 
+    /// <summary>
+    /// Sets the compressor for the message body
+    /// </summary>
+    void set_compressor(std::unique_ptr<http::compression::compress_provider> compressor)
+    {
+        m_compressor = std::move(compressor);
+    }
+
+    /// <summary>
+    /// Gets the compressor for the message body, if any
+    /// </summary>
+    std::unique_ptr<http::compression::compress_provider> &compressor()
+    {
+        return m_compressor;
+    }
+
+    /// <summary>
+    /// Sets the collection of factory classes for decompressors for use with the message body
+    /// </summary>
+    void set_decompress_factories(const std::vector<std::shared_ptr<http::compression::decompress_factory>> &factories)
+    {
+        m_decompressors = factories;
+    }
+
+    /// <summary>
+    /// Gets the collection of factory classes for decompressors to be used to decompress the message body, if any
+    /// </summary>
+    const std::vector<std::shared_ptr<http::compression::decompress_factory>> &decompress_factories()
+    {
+        return m_decompressors;
+    }
+
     const pplx::task_completion_event<utility::size64_t> & _get_data_available() const { return m_data_available; }
 
     /// <summary>
@@ -345,11 +378,25 @@ public:
     /// </summary>
     _ASYNCRTIMP void _prepare_to_receive_data();
 
+
+    /// <summary>
+    /// Determine the remaining input stream length
+    /// </summary>
+    /// <returns>
+    /// std::numeric_limits<size_t>::max() if the stream's remaining length cannot be determined
+    /// length      if the stream's remaining length (which may be 0) can be determined
+    /// </returns>
+    /// <remarks>
+    /// This routine should only be called after a msg (request/response) has been
+    /// completely constructed.
+    /// </remarks>
+    _ASYNCRTIMP size_t _get_stream_length();
+
     /// <summary>
     /// Determine the content length
     /// </summary>
     /// <returns>
-    /// size_t::max if there is content with unknown length (transfer_encoding:chunked)
+    /// std::numeric_limits<size_t>::max() if there is content with unknown length (transfer_encoding:chunked)
     /// 0           if there is no content
     /// length      if there is content with known length
     /// </returns>
@@ -359,14 +406,33 @@ public:
     /// </remarks>
     _ASYNCRTIMP size_t _get_content_length();
 
+    /// <summary>
+    /// Determine the content length, and, if necessary, manage compression in the Transfer-Encoding header
+    /// </summary>
+    /// <returns>
+    /// std::numeric_limits<size_t>::max() if there is content with unknown length (transfer_encoding:chunked)
+    /// 0           if there is no content
+    /// length      if there is content with known length
+    /// </returns>
+    /// <remarks>
+    /// This routine is like _get_content_length, except that it adds a compression algorithm to
+    /// the Trasfer-Length header if compression is configured.  It throws if a Transfer-Encoding
+    /// header exists and does not match the one it generated.
+    /// </remarks>
+    _ASYNCRTIMP size_t _get_content_length_and_set_compression();
+
 protected:
 
+    std::unique_ptr<http::compression::compress_provider> m_compressor;
+    std::unique_ptr<http::compression::decompress_provider> m_decompressor;
+    std::vector<std::shared_ptr<http::compression::decompress_factory>> m_decompressors;
+
     /// <summary>
     /// Stream to read the message body.
     /// By default this is an invalid stream. The user could set the instream on
     /// a request by calling set_request_stream(...). This would also be set when
     /// set_body() is called - a stream from the body is constructed and set.
-    /// Even in the presense of msg body this stream could be invalid. An example
+    /// Even in the presence of msg body this stream could be invalid. An example
     /// would be when the user sets an ostream for the response. With that API the
     /// user does not provide the ability to read the msg body.
     /// Thus m_instream is valid when there is a msg body and it can actually be read
@@ -386,6 +452,8 @@ protected:
 
     /// <summary> The TCE is used to signal the availability of the message body. </summary>
     pplx::task_completion_event<utility::size64_t> m_data_available;
+
+    size_t _get_content_length(bool honor_compression);
 };
 
 /// <summary>
@@ -671,7 +739,7 @@ public:
     /// <param name="stream">A readable, open asynchronous stream.</param>
     /// <param name="content_type">A string holding the MIME type of the message body.</param>
     /// <remarks>
-    /// This cannot be used in conjunction with any other means of setting the body of the request.
+    /// This cannot be used in conjunction with any external means of setting the body of the request.
     /// The stream will not be read until the message is sent.
     /// </remarks>
     void set_body(const concurrency::streams::istream &stream, const utility::string_t &content_type = _XPLATSTR("application/octet-stream"))
@@ -687,7 +755,7 @@ public:
     /// <param name="content_length">The size of the data to be sent in the body.</param>
     /// <param name="content_type">A string holding the MIME type of the message body.</param>
     /// <remarks>
-    /// This cannot be used in conjunction with any other means of setting the body of the request.
+    /// This cannot be used in conjunction with any external means of setting the body of the request.
     /// The stream will not be read until the message is sent.
     /// </remarks>
     void set_body(const concurrency::streams::istream &stream, utility::size64_t content_length, const utility::string_t &content_type = _XPLATSTR("application/octet-stream"))
@@ -810,6 +878,9 @@ private:
     http::method m_method;
 
     // Tracks whether or not a response has already been started for this message.
+    // 0 = No reply sent
+    // 1 = Usual reply sent
+    // 2 = Reply aborted by another thread; e.g. server shutdown
     pplx::details::atomic_long m_initiated_response;
 
     std::unique_ptr<http::details::_http_server_context> m_server_context;
@@ -1149,6 +1220,94 @@ public:
         return _m_impl->set_response_stream(stream);
     }
 
+    /// <summary>
+    /// Sets a compressor that will be used to compress the body of the HTTP message as it is sent.
+    /// </summary>
+    /// <param name="compressor">A pointer to an instantiated compressor of the desired type.</param>
+    /// <remarks>
+    /// This cannot be used in conjunction with any external means of compression.  The Transfer-Encoding
+    /// header will be managed internally, and must not be set by the client.
+    /// </remarks>
+    void set_compressor(std::unique_ptr<http::compression::compress_provider> compressor)
+    {
+        return _m_impl->set_compressor(std::move(compressor));
+    }
+
+    /// <summary>
+    /// Sets a compressor that will be used to compress the body of the HTTP message as it is sent.
+    /// </summary>
+    /// <param name="algorithm">The built-in compression algorithm to use.</param>
+    /// <returns>
+    /// True if a built-in compressor was instantiated, otherwise false.
+    /// </returns>
+    /// <remarks>
+    /// This cannot be used in conjunction with any external means of compression.  The Transfer-Encoding
+    /// header will be managed internally, and must not be set by the client.
+    /// </remarks>
+    bool set_compressor(utility::string_t algorithm)
+    {
+        _m_impl->set_compressor(http::compression::builtin::make_compressor(algorithm));
+        return (bool)_m_impl->compressor();
+    }
+
+    /// <summary>
+    /// Gets the compressor to be used to compress the message body, if any.
+    /// </summary>
+    /// <returns>
+    /// The compressor itself.
+    /// </returns>
+    std::unique_ptr<http::compression::compress_provider> &compressor()
+    {
+        return _m_impl->compressor();
+    }
+
+    /// <summary>
+    /// Sets the default collection of built-in factory classes for decompressors that may be used to
+    /// decompress the body of the HTTP message as it is received, effectively enabling decompression.
+    /// </summary>
+    /// <param name="factories">The collection of factory classes for allowable decompressors. The
+    /// supplied vector itself need not remain valid after the call returns.</param>
+    /// <remarks>
+    /// This default collection is implied if request_compressed_response() is set in the associated
+    /// <c>client::http_client_config</c> and neither overload of this method has been called.
+    ///
+    /// This cannot be used in conjunction with any external means of decompression.  The TE and Accept-Encoding
+    /// headers must not be set by the client, as they will be managed internally as appropriate.
+    /// </remarks>
+    _ASYNCRTIMP void set_decompress_factories();
+
+    /// <summary>
+    /// Sets a collection of factory classes for decompressors that may be used to decompress the
+    /// body of the HTTP message as it is received, effectively enabling decompression.
+    /// </summary>
+    /// <remarks>
+    /// If set, this collection takes the place of the built-in compression providers.  It may contain
+    /// custom factory classes and/or factory classes for built-in providers, and may be used to adjust
+    /// the weights of the built-in providers, which default to 500 (i.e. "q=0.500").
+    ///
+    /// This cannot be used in conjunction with any external means of decompression.  The TE and Accept-Encoding
+    /// headers must not be set by the client, as they will be managed internally as appropriate.
+    /// </remarks>
+    void set_decompress_factories(const std::vector<std::shared_ptr<http::compression::decompress_factory>> &factories)
+    {
+        return _m_impl->set_decompress_factories(factories);
+    }
+
+    /// <summary>
+    /// Gets the collection of factory classes for decompressors to be used to decompress the message body, if any.
+    /// </summary>
+    /// <returns>
+    /// The collection of factory classes itself.
+    /// </returns>
+    /// <remarks>
+    /// This cannot be used in conjunction with any external means of decompression.  The TE
+    /// header must not be set by the client, as it will be managed internally.
+    /// </remarks>
+    const std::vector<std::shared_ptr<http::compression::decompress_factory>> &decompress_factories() const
+    {
+        return _m_impl->decompress_factories();
+    }
+
     /// <summary>
     /// Defines a callback function that will be invoked for every chunk of data uploaded or downloaded
     /// as part of the request.
