@@ -12,8 +12,8 @@
 ****/
 #pragma once
 
-#ifndef _CASA_URI_H
-#define _CASA_URI_H
+#ifndef CASA_URI_H
+#define CASA_URI_H
 
 #include "cpprest/base_uri.h"
 #include "cpprest/uri_builder.h"
