@@ -14,8 +14,10 @@
  ****/
 
 #include "stdafx.h"
+#include <sstream>
 
 #include "../common/internal_http_helpers.h"
+#include "../common/connection_pool_helpers.h"
 #include "cpprest/asyncrt_utils.h"
 
 #if defined(__clang__)
@@ -344,48 +346,6 @@ private:
     bool m_closed;
 };
 
-class connection_pool_stack
-{
-public:
-    // attempts to acquire a connection from the deque; returns nullptr if no connection is
-    // available
-    std::shared_ptr<asio_connection> try_acquire() CPPREST_NOEXCEPT
-    {
-        const size_t oldConnectionsSize = m_connections.size();
-        if (m_highWater > oldConnectionsSize)
-        {
-            m_highWater = oldConnectionsSize;
-        }
-
-        if (oldConnectionsSize == 0)
-        {
-            return nullptr;
-        }
-
-        auto result = std::move(m_connections.back());
-        m_connections.pop_back();
-        return result;
-    }
-
-    // releases `released` back to the connection pool
-    void release(std::shared_ptr<asio_connection>&& released)
-    {
-        m_connections.push_back(std::move(released));
-    }
-
-    bool free_stale_connections() CPPREST_NOEXCEPT
-    {
-        m_connections.erase(m_connections.begin(), m_connections.begin() + m_highWater);
-        const size_t connectionsSize = m_connections.size();
-        m_highWater = connectionsSize;
-        return (connectionsSize != 0);
-    }
-
-private:
-    size_t m_highWater = 0;
-    std::vector<std::shared_ptr<asio_connection>> m_connections;
-};
-
 /// <summary>Implements a connection pool with adaptive connection removal</summary>
 /// <remarks>
 /// Every 30 seconds, the lambda in `start_epoch_interval` fires, triggering the
@@ -500,7 +460,7 @@ private:
     }
 
     std::mutex m_lock;
-    std::map<std::string, connection_pool_stack> m_connections;
+    std::map<std::string, connection_pool_stack<asio_connection>> m_connections;
     bool m_is_timer_running;
     boost::asio::deadline_timer m_pool_epoch_timer;
 };
@@ -850,12 +810,12 @@ public:
                 extra_headers.append(ctx->generate_basic_auth_header());
             }
 
-            extra_headers += utility::conversions::to_utf8string(ctx->get_accept_encoding_header());
+            extra_headers += utility::conversions::to_utf8string(ctx->get_compression_header());
 
             // Check user specified transfer-encoding.
             std::string transferencoding;
             if (ctx->m_request.headers().match(header_names::transfer_encoding, transferencoding) &&
-                transferencoding == "chunked")
+                boost::icontains(transferencoding, U("chunked")))
             {
                 ctx->m_needChunked = true;
             }
@@ -1394,7 +1354,7 @@ private:
 
                 if (boost::iequals(name, header_names::transfer_encoding))
                 {
-                    needChunked = boost::iequals(value, U("chunked"));
+                    needChunked = boost::icontains(value, U("chunked"));
                 }
 
                 if (boost::iequals(name, header_names::connection))
@@ -1415,7 +1375,7 @@ private:
                                                                // TCP stream - set it size_t max.
         m_response.headers().match(header_names::content_length, m_content_length);
 
-        if (!this->handle_content_encoding_compression())
+        if (!this->handle_compression())
         {
             // false indicates report_exception was called
             return;
@@ -1518,6 +1478,49 @@ private:
         }
     }
 
+    bool decompress(const uint8_t* input, size_t input_size, std::vector<uint8_t>& output)
+    {
+        // Need to guard against attempting to decompress when we're already finished or encountered an error!
+        if (input == nullptr || input_size == 0)
+        {
+            return false;
+        }
+
+        size_t processed;
+        size_t got;
+        size_t inbytes = 0;
+        size_t outbytes = 0;
+        bool done;
+
+        try
+        {
+            output.resize(input_size * 3);
+            do
+            {
+                if (inbytes)
+                {
+                    output.resize(output.size() + std::max(input_size, static_cast<size_t>(1024)));
+                }
+                got = m_decompressor->decompress(input + inbytes,
+                                                 input_size - inbytes,
+                                                 output.data() + outbytes,
+                                                 output.size() - outbytes,
+                                                 web::http::compression::operation_hint::has_more,
+                                                 processed,
+                                                 done);
+                inbytes += processed;
+                outbytes += got;
+            } while (got && !done);
+            output.resize(outbytes);
+        }
+        catch (...)
+        {
+            return false;
+        }
+
+        return true;
+    }
+
     void handle_chunk(const boost::system::error_code& ec, int to_read)
     {
         if (!ec)
@@ -1550,10 +1553,11 @@ private:
                 const auto this_request = shared_from_this();
                 if (m_decompressor)
                 {
-                    auto decompressed = m_decompressor->decompress(
-                        boost::asio::buffer_cast<const uint8_t*>(m_body_buf.data()), to_read);
+                    std::vector<uint8_t> decompressed;
 
-                    if (m_decompressor->has_error())
+                    bool boo =
+                        decompress(boost::asio::buffer_cast<const uint8_t*>(m_body_buf.data()), to_read, decompressed);
+                    if (!boo)
                     {
                         report_exception(std::runtime_error("Failed to decompress the response body"));
                         return;
@@ -1574,8 +1578,7 @@ private:
                     {
                         // Move the decompressed buffer into a shared_ptr to keep it alive until putn_nocopy completes.
                         // When VS 2013 support is dropped, this should be changed to a unique_ptr plus a move capture.
-                        using web::http::details::compression::data_buffer;
-                        auto shared_decompressed = std::make_shared<data_buffer>(std::move(decompressed));
+                        auto shared_decompressed = std::make_shared<std::vector<uint8_t>>(std::move(decompressed));
 
                         writeBuffer.putn_nocopy(shared_decompressed->data(), shared_decompressed->size())
                             .then([this_request, to_read, shared_decompressed AND_CAPTURE_MEMBER_FUNCTION_POINTERS](
@@ -1670,10 +1673,11 @@ private:
 
             if (m_decompressor)
             {
-                auto decompressed =
-                    m_decompressor->decompress(boost::asio::buffer_cast<const uint8_t*>(m_body_buf.data()), read_size);
+                std::vector<uint8_t> decompressed;
 
-                if (m_decompressor->has_error())
+                bool boo =
+                    decompress(boost::asio::buffer_cast<const uint8_t*>(m_body_buf.data()), read_size, decompressed);
+                if (!boo)
                 {
                     this_request->report_exception(std::runtime_error("Failed to decompress the response body"));
                     return;
@@ -1704,18 +1708,18 @@ private:
                 {
                     // Move the decompressed buffer into a shared_ptr to keep it alive until putn_nocopy completes.
                     // When VS 2013 support is dropped, this should be changed to a unique_ptr plus a move capture.
-                    using web::http::details::compression::data_buffer;
-                    auto shared_decompressed = std::make_shared<data_buffer>(std::move(decompressed));
+                    auto shared_decompressed = std::make_shared<std::vector<uint8_t>>(std::move(decompressed));
 
                     writeBuffer.putn_nocopy(shared_decompressed->data(), shared_decompressed->size())
                         .then([this_request, read_size, shared_decompressed AND_CAPTURE_MEMBER_FUNCTION_POINTERS](
                                   pplx::task<size_t> op) {
                             size_t writtenSize = 0;
+                            (void)writtenSize;
                             try
                             {
                                 writtenSize = op.get();
                                 this_request->m_downloaded += static_cast<uint64_t>(read_size);
-                                this_request->m_body_buf.consume(writtenSize);
+                                this_request->m_body_buf.consume(read_size);
                                 this_request->async_read_until_buffersize(
                                     static_cast<size_t>(std::min(
                                         static_cast<uint64_t>(this_request->m_http_client->client_config().chunksize()),
@@ -1838,7 +1842,7 @@ private:
             timedout
         };
 
-#if defined(ANDROID) || defined(__ANDROID__)
+#if (defined(ANDROID) || defined(__ANDROID__)) && !defined(_LIBCPP_VERSION)
         boost::chrono::microseconds m_duration;
 #else
         std::chrono::microseconds m_duration;
