@@ -11,7 +11,9 @@
 * =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ****/
 #include "stdafx.h"
+#include <sstream>
 #include "../common/internal_http_helpers.h"
+#include "cpprest/producerconsumerstream.h"
 
 using namespace web;
 using namespace utility;
@@ -253,7 +255,7 @@ void parse_headers_string(_Inout_z_ utf16char *headersStr, http_headers &headers
 
 http_version __cdecl http_version::from_string(const std::string& http_version_string)
 {
-    std::stringstream str(http_version_string);
+    std::istringstream str(http_version_string);
     str.imbue(std::locale::classic());
 
     std::string http; std::getline(str, http, '/');
@@ -310,36 +312,95 @@ void http_msg_base::_prepare_to_receive_data()
     // or media (like file) that the user can read from...
 }
 
-size_t http_msg_base::_get_content_length()
+size_t http_msg_base::_get_stream_length()
+{
+    auto &stream = instream();
+
+    if (stream.can_seek())
+    {
+        auto offset = stream.tell();
+        auto end = stream.seek(0, std::ios_base::end);
+        stream.seek(offset);
+        return static_cast<size_t>(end - offset);
+    }
+
+    return std::numeric_limits<size_t>::max();
+}
+
+size_t http_msg_base::_get_content_length(bool honor_compression)
 {
     // An invalid response_stream indicates that there is no body
     if ((bool)instream())
     {
-        size_t content_length = 0;
+        size_t content_length;
         utility::string_t transfer_encoding;
 
-        bool has_cnt_length = headers().match(header_names::content_length, content_length);
-        bool has_xfr_encode = headers().match(header_names::transfer_encoding, transfer_encoding);
-
-        if (has_xfr_encode)
+        if (headers().match(header_names::transfer_encoding, transfer_encoding))
         {
+            // Transfer encoding is set; it trumps any content length that may or may not be present
+            if (honor_compression && m_compressor)
+            {
+                http::http_headers tmp;
+
+                // Build a header for comparison with the existing one
+                tmp.add(header_names::transfer_encoding, m_compressor->algorithm());
+                tmp.add(header_names::transfer_encoding, _XPLATSTR("chunked"));
+
+                if (!utility::details::str_iequal(transfer_encoding, tmp[header_names::transfer_encoding]))
+                {
+                    // Some external entity added this header, and it doesn't match our
+                    // expectations; bail out, since the caller's intentions are not clear
+                    throw http_exception("Transfer-Encoding header is internally managed when compressing");
+                }
+            }
+
             return std::numeric_limits<size_t>::max();
         }
 
-        if (has_cnt_length)
+        if (honor_compression && m_compressor)
         {
+            // A compressor is set; this implies transfer encoding, since we don't know the compressed length
+            // up front for content encoding.  We return the uncompressed length if we can figure it out.
+            headers().add(header_names::transfer_encoding, m_compressor->algorithm());
+            headers().add(header_names::transfer_encoding, _XPLATSTR("chunked"));
+            return std::numeric_limits<size_t>::max();
+        }
+
+        if (headers().match(header_names::content_length, content_length))
+        {
+            // An explicit content length is set; trust it, since we
+            // may not be required to send the stream's entire contents
             return content_length;
         }
 
-        // Neither is set. Assume transfer-encoding for now (until we have the ability to determine
-        // the length of the stream).
+        content_length = _get_stream_length();
+        if (content_length != std::numeric_limits<size_t>::max())
+        {
+            // The content length wasn't explicitly set, but we figured it out;
+            // use it, since sending this way is more efficient than chunking
+            headers().add(header_names::content_length, content_length);
+            return content_length;
+        }
+
+        // We don't know the content length; we'll chunk the stream
         headers().add(header_names::transfer_encoding, _XPLATSTR("chunked"));
         return std::numeric_limits<size_t>::max();
     }
 
+    // There is no content
     return 0;
 }
 
+size_t http_msg_base::_get_content_length_and_set_compression()
+{
+    return _get_content_length(true);
+}
+
+size_t http_msg_base::_get_content_length()
+{
+    return _get_content_length(false);
+}
+
 // Helper function to inline continuation if possible.
 struct inline_continuation
 {
@@ -950,22 +1011,24 @@ static utility::string_t convert_body_to_string_t(const utility::string_t &conte
 //
 static utility::string_t http_headers_body_to_string(const http_headers &headers, concurrency::streams::istream instream)
 {
-    utility::ostringstream_t buffer;
-    buffer.imbue(std::locale::classic());
-
+    utility::string_t result;
     for (const auto &header : headers)
     {
-        buffer << header.first << _XPLATSTR(": ") << header.second << CRLF;
+        result += header.first;
+        result += _XPLATSTR(": ");
+        result += header.second;
+        result += CRLF;
     }
-    buffer << CRLF;
+
+    result += CRLF;
 
     utility::string_t content_type;
     if(headers.match(http::header_names::content_type, content_type))
     {
-        buffer << convert_body_to_string_t(content_type, instream);
+        result += convert_body_to_string_t(content_type, instream);
     }
 
-    return buffer.str();
+    return result;
 }
 
 utility::string_t details::http_msg_base::to_string() const
@@ -1041,6 +1104,11 @@ details::_http_request::_http_request(std::unique_ptr<http::details::_http_serve
 {
 }
 
+void http_request::set_decompress_factories()
+{
+    return _m_impl->set_decompress_factories(compression::details::builtin::get_decompress_factories());
+}
+
 const http_version http_versions::HTTP_0_9 = { 0, 9 };
 const http_version http_versions::HTTP_1_0 = { 1, 0 };
 const http_version http_versions::HTTP_1_1 = { 1, 1 };
