@@ -14,6 +14,8 @@
 */
 #include "stdafx.h"
 
+#include <sstream>
+#include <set>
 #include <boost/algorithm/string/find.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/asio/read_until.hpp>
@@ -248,7 +250,7 @@ struct crlfcrlf_nonascii_searcher_t
         State state = State::none;
         while (cur != end)
         {
-            char c = *cur;
+            const auto c = static_cast<unsigned char>(*cur);
             if (c == '\r')
             {
                 if (state == State::crlf)
@@ -280,12 +282,7 @@ struct crlfcrlf_nonascii_searcher_t
                     state = State::none;
                 }
             }
-            else if (c <= '\x1F' && c >= '\x00')
-            {
-                ++cur;
-                return std::make_pair(cur, true);
-            }
-            else if (c <= '\xFF' && c >= '\x80')
+            else if (c <= 0x1Fu || c >= 0x80)
             {
                 ++cur;
                 return std::make_pair(cur, true);
@@ -318,7 +315,19 @@ private:
     boost::asio::streambuf m_response_buf;
     http_linux_server* m_p_server;
     hostport_listener* m_p_parent;
-    http_request m_request;
+    mutable std::mutex m_request_mtx;
+    http_request m_request_tmp;
+
+    void set_request(http_request req) {
+        std::lock_guard<std::mutex> lck(m_request_mtx);
+        m_request_tmp = std::move(req);
+    }
+
+    http_request get_request() const {
+        std::lock_guard<std::mutex> lck(m_request_mtx);
+        return m_request_tmp;
+    }
+
     size_t m_read, m_write;
     size_t m_read_size, m_write_size;
     bool m_close;
@@ -407,7 +416,7 @@ private:
     will_erase_from_parent_t do_response()
     {
         auto unique_reference = this->get_reference();
-        m_request.get_response().then([=](pplx::task<http_response> r_task)
+        get_request().get_response().then([=](pplx::task<http_response> r_task)
         {
             http_response response;
             try
@@ -422,7 +431,7 @@ private:
             serialize_headers(response);
 
             // before sending response, the full incoming message need to be processed.
-            return m_request.content_ready().then([=](pplx::task<http_request>)
+            return get_request().content_ready().then([=](pplx::task<http_request>)
             {
                 (will_deref_and_erase_t)this->async_write(&asio_server_connection::handle_headers_written, response);
             });
@@ -433,7 +442,7 @@ private:
     will_erase_from_parent_t do_bad_response()
     {
         auto unique_reference = this->get_reference();
-        m_request.get_response().then([=](pplx::task<http_response> r_task)
+        get_request().get_response().then([=](pplx::task<http_response> r_task)
         {
             http_response response;
             try
@@ -538,7 +547,7 @@ void asio_server_connection::close()
         sock->shutdown(tcp::socket::shutdown_both, ec);
         sock->close(ec);
     }
-    m_request._reply_if_not_already(status_codes::InternalError);
+    get_request()._reply_if_not_already(status_codes::InternalError);
 }
 
 will_deref_and_erase_t asio_server_connection::start_request_response()
@@ -615,7 +624,8 @@ void hostport_listener::on_accept(std::unique_ptr<ip::tcp::socket> socket, const
 
 will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::system::error_code& ec)
 {
-    m_request = http_request::_create_request(make_unique<linux_request_context>());
+    auto thisRequest = http_request::_create_request(make_unique<linux_request_context>());
+    set_request(thisRequest);
     if (ec)
     {
         // client closed connection
@@ -630,7 +640,7 @@ will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::sys
         }
         else
         {
-            m_request._reply_if_not_already(status_codes::BadRequest);
+            thisRequest._reply_if_not_already(status_codes::BadRequest);
             m_close = true;
             (will_erase_from_parent_t)do_bad_response();
             (will_deref_t)deref();
@@ -667,14 +677,14 @@ will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::sys
         // Check to see if there is not allowed character on the input
         if (!web::http::details::validate_method(http_verb))
         {
-            m_request.reply(status_codes::BadRequest);
+            thisRequest.reply(status_codes::BadRequest);
             m_close = true;
             (will_erase_from_parent_t)do_bad_response();
             (will_deref_t)deref();
             return will_deref_and_erase_t{};
         }
 
-        m_request.set_method(http_verb);
+        thisRequest.set_method(http_verb);
 
         std::string http_path_and_version;
         std::getline(request_stream, http_path_and_version);
@@ -683,7 +693,7 @@ will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::sys
         // Make sure path and version is long enough to contain the HTTP version
         if(http_path_and_version.size() < VersionPortionSize + 2)
         {
-            m_request.reply(status_codes::BadRequest);
+            thisRequest.reply(status_codes::BadRequest);
             m_close = true;
             (will_erase_from_parent_t)do_bad_response();
             (will_deref_t)deref();
@@ -693,11 +703,12 @@ will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::sys
         // Get the path - remove the version portion and prefix space
         try
         {
-            m_request.set_request_uri(utility::conversions::to_string_t(http_path_and_version.substr(1, http_path_and_version.size() - VersionPortionSize - 1)));
+            thisRequest.set_request_uri(utility::conversions::to_string_t(
+                http_path_and_version.substr(1, http_path_and_version.size() - VersionPortionSize - 1)));
         }
         catch (const std::exception& e) // may be std::range_error indicating invalid Unicode, or web::uri_exception
         {
-            m_request.reply(status_codes::BadRequest, e.what());
+            thisRequest.reply(status_codes::BadRequest, e.what());
             m_close = true;
             (will_erase_from_parent_t)do_bad_response();
             (will_deref_t)deref();
@@ -707,9 +718,9 @@ will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::sys
         // Get the version
         std::string http_version = http_path_and_version.substr(http_path_and_version.size() - VersionPortionSize + 1, VersionPortionSize - 2);
 
-        auto m_request_impl = m_request._get_impl().get();
+        auto requestImpl = thisRequest._get_impl().get();
         web::http::http_version parsed_version = web::http::http_version::from_string(http_version);
-        m_request_impl->_set_http_version(parsed_version);
+        requestImpl->_set_http_version(parsed_version);
 
         // if HTTP version is 1.0 then disable pipelining
         if (parsed_version == web::http::http_versions::HTTP_1_0)
@@ -722,7 +733,8 @@ will_deref_and_erase_t asio_server_connection::handle_http_line(const boost::sys
         auto endpoint = m_socket->remote_endpoint(socket_ec);
         if (!socket_ec)
         {
-            m_request._get_impl()->_set_remote_address(utility::conversions::to_string_t(endpoint.address().to_string()));
+            requestImpl->_set_remote_address(utility::conversions::to_string_t(
+                endpoint.address().to_string()));
         }
 
         return handle_headers();
@@ -735,7 +747,8 @@ will_deref_and_erase_t asio_server_connection::handle_headers()
     request_stream.imbue(std::locale::classic());
     std::string header;
 
-    auto& headers = m_request.headers();
+    auto currentRequest = get_request();
+    auto& headers = currentRequest.headers();
 
     while (std::getline(request_stream, header) && header != "\r")
     {
@@ -743,7 +756,8 @@ will_deref_and_erase_t asio_server_connection::handle_headers()
         if (colon != std::string::npos && colon != 0)
         {
             auto name = utility::conversions::to_string_t(header.substr(0, colon));
-            auto value = utility::conversions::to_string_t(header.substr(colon + 1, header.length() - (colon + 1))); // also exclude '\r'
+            auto value = utility::conversions::to_string_t(
+                header.substr(colon + 1, header.length() - (colon + 1))); // also exclude '\r'
             web::http::details::trim_whitespace(name);
             web::http::details::trim_whitespace(value);
 
@@ -758,7 +772,7 @@ will_deref_and_erase_t asio_server_connection::handle_headers()
         }
         else
         {
-            m_request.reply(status_codes::BadRequest);
+            currentRequest.reply(status_codes::BadRequest);
             m_close = true;
             (will_erase_from_parent_t)do_bad_response();
             (will_deref_t)deref();
@@ -769,17 +783,17 @@ will_deref_and_erase_t asio_server_connection::handle_headers()
     m_chunked = false;
     utility::string_t name;
     // check if the client has requested we close the connection
-    if (m_request.headers().match(header_names::connection, name))
+    if (currentRequest.headers().match(header_names::connection, name))
     {
         m_close = boost::iequals(name, U("close"));
     }
 
-    if (m_request.headers().match(header_names::transfer_encoding, name))
+    if (currentRequest.headers().match(header_names::transfer_encoding, name))
     {
         m_chunked = boost::ifind_first(name, U("chunked"));
     }
 
-    m_request._get_impl()->_prepare_to_receive_data();
+    currentRequest._get_impl()->_prepare_to_receive_data();
     if (m_chunked)
     {
         ++m_refs;
@@ -787,14 +801,14 @@ will_deref_and_erase_t asio_server_connection::handle_headers()
         return dispatch_request_to_listener();
     }
 
-    if (!m_request.headers().match(header_names::content_length, m_read_size))
+    if (!currentRequest.headers().match(header_names::content_length, m_read_size))
     {
         m_read_size = 0;
     }
 
     if (m_read_size == 0)
     {
-        m_request._get_impl()->_complete(0);
+        currentRequest._get_impl()->_complete(0);
     }
     else // need to read the sent data
     {
@@ -811,9 +825,10 @@ will_deref_and_erase_t asio_server_connection::handle_headers()
 
 will_deref_t asio_server_connection::handle_chunked_header(const boost::system::error_code& ec)
 {
+    auto requestImpl = get_request()._get_impl();
     if (ec)
     {
-        m_request._get_impl()->_complete(0, std::make_exception_ptr(http_exception(ec.value())));
+        requestImpl->_complete(0, std::make_exception_ptr(http_exception(ec.value())));
         return deref();
     }
     else
@@ -826,7 +841,7 @@ will_deref_t asio_server_connection::handle_chunked_header(const boost::system::
         m_read += len;
         if (len == 0)
         {
-            m_request._get_impl()->_complete(m_read);
+            requestImpl->_complete(m_read);
             return deref();
         }
         else
@@ -842,14 +857,15 @@ will_deref_t asio_server_connection::handle_chunked_header(const boost::system::
 
 will_deref_t asio_server_connection::handle_chunked_body(const boost::system::error_code& ec, int toWrite)
 {
+    auto requestImpl = get_request()._get_impl();
     if (ec)
     {
-        m_request._get_impl()->_complete(0, std::make_exception_ptr(http_exception(ec.value())));
+        requestImpl->_complete(0, std::make_exception_ptr(http_exception(ec.value())));
         return deref();
     }
     else
     {
-        auto writebuf = m_request._get_impl()->outstream().streambuf();
+        auto writebuf = requestImpl->outstream().streambuf();
         writebuf.putn_nocopy(buffer_cast<const uint8_t *>(m_request_buf.data()), toWrite).then([=](pplx::task<size_t> writeChunkTask) -> will_deref_t
         {
             try
@@ -858,7 +874,7 @@ will_deref_t asio_server_connection::handle_chunked_body(const boost::system::er
             }
             catch (...)
             {
-                m_request._get_impl()->_complete(0, std::current_exception());
+                requestImpl->_complete(0, std::current_exception());
                 return deref();
             }
 
@@ -871,16 +887,17 @@ will_deref_t asio_server_connection::handle_chunked_body(const boost::system::er
 
 will_deref_t asio_server_connection::handle_body(const boost::system::error_code& ec)
 {
+    auto requestImpl = get_request()._get_impl();
     // read body
     if (ec)
     {
-        m_request._get_impl()->_complete(0, std::make_exception_ptr(http_exception(ec.value())));
+        requestImpl->_complete(0, std::make_exception_ptr(http_exception(ec.value())));
         return deref();
     }
     else if (m_read < m_read_size)  // there is more to read
     {
-        auto writebuf = m_request._get_impl()->outstream().streambuf();
-        writebuf.putn_nocopy(boost::asio::buffer_cast<const uint8_t*>(m_request_buf.data()), std::min(m_request_buf.size(), m_read_size - m_read)).then([=](pplx::task<size_t> writtenSizeTask) -> will_deref_t
+        auto writebuf = requestImpl->outstream().streambuf();
+        writebuf.putn_nocopy(boost::asio::buffer_cast<const uint8_t*>(m_request_buf.data()), std::min(m_request_buf.size(), m_read_size - m_read)).then([this](pplx::task<size_t> writtenSizeTask) -> will_deref_t
         {
             size_t writtenSize = 0;
             try
@@ -889,7 +906,7 @@ will_deref_t asio_server_connection::handle_body(const boost::system::error_code
             }
             catch (...)
             {
-                m_request._get_impl()->_complete(0, std::current_exception());
+                get_request()._get_impl()->_complete(0, std::current_exception());
                 return deref();
             }
             m_read += writtenSize;
@@ -905,7 +922,7 @@ will_deref_t asio_server_connection::handle_body(const boost::system::error_code
     }
     else  // have read request body
     {
-        m_request._get_impl()->_complete(m_read);
+        requestImpl->_complete(m_read);
         return deref();
     }
 }
@@ -976,14 +993,14 @@ will_deref_and_erase_t asio_server_connection::dispatch_request_to_listener()
 {
     // locate the listener:
     http_listener_impl* pListener = nullptr;
-
+    auto currentRequest = get_request();
     try
     {
-        pListener = m_p_parent->find_listener(m_request.relative_uri());
+        pListener = m_p_parent->find_listener(currentRequest.relative_uri());
     }
     catch (const std::exception&) // may be web::uri_exception, or std::range_error indicating invalid Unicode
     {
-        m_request.reply(status_codes::BadRequest);
+        currentRequest.reply(status_codes::BadRequest);
         (will_erase_from_parent_t)do_response();
         (will_deref_t)deref();
         return will_deref_and_erase_t{};
@@ -991,13 +1008,13 @@ will_deref_and_erase_t asio_server_connection::dispatch_request_to_listener()
 
     if (pListener == nullptr)
     {
-        m_request.reply(status_codes::NotFound);
+        currentRequest.reply(status_codes::NotFound);
         (will_erase_from_parent_t)do_response();
         (will_deref_t)deref();
         return will_deref_and_erase_t{};
     }
 
-    m_request._set_listener_path(pListener->uri().path());
+    currentRequest._set_listener_path(pListener->uri().path());
     (will_erase_from_parent_t)do_response();
 
     // Look up the lock for the http_listener.
@@ -1008,7 +1025,7 @@ will_deref_and_erase_t asio_server_connection::dispatch_request_to_listener()
         // It is possible the listener could have unregistered.
         if(m_p_server->m_registered_listeners.find(pListener) == m_p_server->m_registered_listeners.end())
         {
-            m_request.reply(status_codes::NotFound);
+            currentRequest.reply(status_codes::NotFound);
 
             (will_deref_t)deref();
             return will_deref_and_erase_t{};
@@ -1022,13 +1039,13 @@ will_deref_and_erase_t asio_server_connection::dispatch_request_to_listener()
 
     try
     {
-        pListener->handle_request(m_request);
+        pListener->handle_request(currentRequest);
         pListenerLock->unlock();
     }
     catch(...)
     {
         pListenerLock->unlock();
-        m_request._reply_if_not_already(status_codes::InternalError);
+        currentRequest._reply_if_not_already(status_codes::InternalError);
     }
 
     (will_deref_t)deref();
@@ -1325,7 +1342,7 @@ pplx::task<void> http_linux_server::register_listener(http_listener_impl* listen
         }
         catch (...)
         {
-            // Future improvement - really this API should entirely be asychronously.
+            // Future improvement - really this API should entirely be asynchronously.
             // the hostport_listener::start() method should be made to return a task
             // throwing the exception.
             m_registered_listeners.erase(listener);
